@@ -15,21 +15,27 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import os, sys
+import os
 import glob
 import locale
 import logging
 import gc
 import genshi
+import sys
 
-from config import LazygalConfig, LazygalWebgalConfig,\
-                   USER_CONFIG_PATH, LazygalConfigDeprecated
+from config import LazygalConfig, LazygalWebgalConfig
+from config import USER_CONFIG_PATH, LazygalConfigDeprecated
 from sourcetree import SOURCEDIR_CONFIGFILE
 
 import make
 import pathutils
-import sourcetree, tpl, newsize, metadata
-import genpage, genmedia, genfile
+import sourcetree
+import tpl
+import newsize
+import metadata
+import genpage
+import genmedia
+import genfile
 
 
 from lazygal import INSTALL_MODE, INSTALL_PREFIX
@@ -42,9 +48,6 @@ elif INSTALL_MODE == 'installed':
         sys.exit(1)
 
 
-THEME_DIR = os.path.join(DATAPATH, 'themes')
-USER_THEME_DIR = os.path.expanduser(os.path.join('~', '.lazygal', 'themes'))
-THEME_SHARED_FILE_PREFIX = 'SHARED_'
 DEST_SHARED_DIRECTORY_NAME = 'shared'
 
 
@@ -62,17 +65,17 @@ class SubgalSort(make.MakeTask):
         logging.info(_("  SORTING pics and subdirs"))
 
         if self.webgal_dir.subgal_sort_by[0] == 'exif':
-            subgal_sorter = lambda x, y:\
-                                x.source_dir.compare_latest_exif(y.source_dir)
+            subgal_sorter = \
+                lambda x, y: x.source_dir.compare_latest_exif(y.source_dir)
         elif self.webgal_dir.subgal_sort_by[0] == 'mtime':
-            subgal_sorter = lambda x, y:\
-                                x.source_dir.compare_mtime(y.source_dir)
+            subgal_sorter = \
+                lambda x, y: x.source_dir.compare_mtime(y.source_dir)
         elif self.webgal_dir.subgal_sort_by[0] == 'dirname'\
-        or self.webgal_dir.subgal_sort_by[0] == 'filename': # Backward compatibility
-            subgal_sorter = lambda x, y:\
-                                x.source_dir.compare_filename(y.source_dir)
+                or self.webgal_dir.subgal_sort_by[0] == 'filename':  # Backward compatibility
+            subgal_sorter = \
+                lambda x, y: x.source_dir.compare_filename(y.source_dir)
         else:
-            raise ValueError(_("Unknown sorting criterion '%s'")\
+            raise ValueError(_("Unknown sorting criterion '%s'")
                              % self.webgal_dir.subgal_sort_by[0])
         self.webgal_dir.subgals.sort(subgal_sorter,
                                      reverse=self.webgal_dir.subgal_sort_by[1])
@@ -84,7 +87,7 @@ class SubgalSort(make.MakeTask):
         elif self.webgal_dir.pic_sort_by[0] == 'filename':
             sorter = lambda x, y: x.media.compare_filename(y.media)
         else:
-            raise ValueError(_("Unknown sorting criterion '%s'")\
+            raise ValueError(_("Unknown sorting criterion '%s'")
                              % self.webgal_dir.pic_sort_by[0])
         self.webgal_dir.medias.sort(sorter,
                                     reverse=self.webgal_dir.pic_sort_by[1])
@@ -142,7 +145,7 @@ class SubgalBreak(make.MakeTask):
         """
         Loose pagination not breaking subgals (chosen if subgals are flattened).
         """
-        subgals = [] # No subgal links as they are flattened
+        subgals = []  # No subgal links as they are flattened
 
         galleries = []
         how_many_medias = 0
@@ -161,8 +164,12 @@ class SubgalBreak(make.MakeTask):
                 self.webgal_dir.add_index_page(subgals, galleries)
 
     def __fill_real_pagination(self):
-        how_many_pages = len(self.webgal_dir.medias)\
-                          // self.webgal_dir.thumbs_per_page + 1
+        medias_amount = len(self.webgal_dir.medias)
+        how_many_pages = medias_amount // self.webgal_dir.thumbs_per_page
+        if medias_amount == 0\
+                or medias_amount % self.webgal_dir.thumbs_per_page > 0:
+            how_many_pages = how_many_pages + 1
+
         for page_number in range(0, how_many_pages):
             step = page_number * self.webgal_dir.thumbs_per_page
             end_index = step + self.webgal_dir.thumbs_per_page
@@ -203,9 +210,10 @@ class WebalbumMediaTask(make.GroupTask):
             if self.webgal.original and not self.webgal.orig_base:
                 self.add_dependency(self.get_original())
 
-            self.browse_pages[size_name] = self.get_browse_page(size_name)
-            if self.webgal.album.force_gen_pages:
-                self.browse_pages[size_name].stamp_delete()
+            if self.webgal.album.theme.kind == 'static':
+                self.browse_pages[size_name] = self.get_browse_page(size_name)
+                if self.webgal.album.force_gen_pages:
+                    self.browse_pages[size_name].stamp_delete()
 
     def set_next(self, media):
         self.next = media
@@ -221,7 +229,7 @@ class WebalbumMediaTask(make.GroupTask):
 
     def get_original_or_symlink(self):
         if not self.webgal.orig_symlink:
-            return genfile.MediaOriginal(self.webgal, self.media)
+            return genfile.CopyMediaOriginal(self.webgal, self.media)
         else:
             return genfile.SymlinkMediaOriginal(self.webgal, self.media)
 
@@ -230,6 +238,13 @@ class WebalbumMediaTask(make.GroupTask):
             self.original = self.get_original_or_symlink()
         return self.original
 
+    def get_browse_page(self, size_name):
+        return genpage.WebalbumBrowsePage(self.webgal, size_name, self)
+
+    def make(self):
+        super(WebalbumMediaTask, self).make()
+        self.webgal.media_done()
+
 
 class WebalbumImageTask(WebalbumMediaTask):
     """
@@ -247,10 +262,14 @@ class WebalbumImageTask(WebalbumMediaTask):
         if self.webgal.newsizers[size_name] == 'original':
             return self.get_original_or_symlink()
         else:
-            return genmedia.ImageOtherSize(self.webgal, self.media, size_name)
-
-    def get_browse_page(self, size_name):
-        return genpage.WebalbumImagePage(self.webgal, size_name, self)
+            sized = genmedia.ImageOtherSize(self.webgal, self.media, size_name)
+            self.media.get_size() # probe size to check if media is broken
+            if not self.media.broken\
+            and sized.get_size() == sized.source_media.get_size():
+                # Do not process if size is the same
+                return self.get_original()
+            else:
+                return sized
 
 
 class WebalbumVideoTask(WebalbumMediaTask):
@@ -268,9 +287,6 @@ class WebalbumVideoTask(WebalbumMediaTask):
 
         self.add_dependency(self.webvideo)
 
-    def get_browse_page(self, size_name):
-        return genpage.WebalbumVideoPage(self.webgal, size_name, self)
-
     def get_resized(self, size_name):
         if not self.webvideo:
             self.webvideo = genmedia.WebVideo(self.webgal, self.media,
@@ -283,7 +299,7 @@ class WebalbumDir(make.FileMakeObject):
     This is a built web gallery with its files, thumbs and reduced pics.
     """
 
-    def __init__(self, dir, subgals, album, album_dest_dir):
+    def __init__(self, dir, subgals, album, album_dest_dir, progress=None):
         self.source_dir = dir
         self.path = os.path.join(album_dest_dir, self.source_dir.strip_root())
         if self.path.endswith(os.sep): self.path = os.path.dirname(self.path)
@@ -301,11 +317,11 @@ class WebalbumDir(make.FileMakeObject):
         self.__configure()
 
         # Create the directory if it does not exist
-        if not os.path.isdir(self.path):
-            logging.info(_("  MKDIR %%WEBALBUMROOT%%/%s")\
+        if not os.path.isdir(self.path) and (self.source_dir.get_media_count() > 0):
+            logging.info(_("  MKDIR %%WEBALBUMROOT%%/%s")
                          % self.source_dir.strip_root())
             logging.debug("(%s)" % self.path)
-            os.makedirs(self.path, mode = 0755)
+            os.makedirs(self.path, mode=0755)
             # Directory did not exist, mark it as so
             self.stamp_delete()
 
@@ -320,7 +336,7 @@ class WebalbumDir(make.FileMakeObject):
             elif media.type == 'video':
                 media_task = WebalbumVideoTask(self, media)
             else:
-                raise NotImplementedError("Unknown media type '%s'"\
+                raise NotImplementedError("Unknown media type '%s'"
                                           % media.type)
             self.medias.append(media_task)
             self.add_dependency(media_task)
@@ -345,6 +361,8 @@ class WebalbumDir(make.FileMakeObject):
         else:
             self.break_task = None
 
+        self.progress = progress
+
     def __parse_browse_sizes(self, sizes_string):
         for single_def in sizes_string.split(','):
             name, string_size = single_def.split('=')
@@ -382,20 +400,24 @@ class WebalbumDir(make.FileMakeObject):
         if self.config.has_section('template-vars'):
             tpl_vars = {}
             for option in self.config.options('template-vars'):
-                value = self.config.get('template-vars', option)
-                value = value.decode(locale.getpreferredencoding())
-                tpl_vars[option] = genshi.core.Markup(value)
+                try:
+                    value = self.config.getboolean('template-vars', option)
+                    tpl_vars[option] = value
+                except ValueError:
+                    value = self.config.get('template-vars', option)
+                    value = value.decode(locale.getpreferredencoding())
+                    tpl_vars[option] = genshi.core.Markup(value)
 
         return tpl_vars
 
     def __configure(self):
-        config_dirs = self.source_dir.parent_paths()[:-1] # strip root dir
-        config_dirs.reverse() # from root to deepest
+        config_dirs = self.source_dir.parent_paths()[:-1]  # strip root dir
+        config_dirs.reverse()  # from root to deepest
         config_files = map(lambda d: os.path.join(d, SOURCEDIR_CONFIGFILE),
                            config_dirs)
         logging.debug(_("  Trying loading gallery configs: %s")
                       % ', '.join(map(self.source_dir.strip_root,
-                                          config_files)))
+                                      config_files)))
         self.config.read(config_files)
 
         self.browse_sizes = []
@@ -406,9 +428,9 @@ class WebalbumDir(make.FileMakeObject):
         self.default_size_name = self.browse_sizes[0]
 
         self.tpl_vars = self.__load_tpl_vars()
-        styles = self.album.get_avail_styles(self.album.theme,
-                                     self.config.get('webgal', 'default-style'))
-        self.tpl_vars.update({'styles' : styles})
+        styles = self.album.theme.get_avail_styles(
+            self.config.get('webgal', 'default-style'))
+        self.tpl_vars.update({'styles': styles})
 
         self.set_original(self.config.getboolean('webgal', 'original'),
                           self.config.getstr('webgal', 'original-baseurl'),
@@ -417,7 +439,7 @@ class WebalbumDir(make.FileMakeObject):
         self.thumbs_per_page = self.config.getint('webgal', 'thumbs-per-page')
 
         if self.config.getboolean('webgal', 'dirzip')\
-        and self.source_dir.get_media_count() > 1:
+                and self.source_dir.get_media_count() > 1:
             self.dirzip = genfile.WebalbumArchive(self)
             self.add_dependency(self.dirzip)
         else:
@@ -432,6 +454,7 @@ class WebalbumDir(make.FileMakeObject):
 
         self.pic_sort_by = self.__parse_sort(self.config.get('webgal', 'sort-medias'))
         self.subgal_sort_by = self.__parse_sort(self.config.get('webgal', 'sort-subgals'))
+        self.filter_by_tag = self.config.get('webgal', 'filter-by-tag')
 
         self.webalbumpic_bg = self.config.get('webgal', 'webalbumpic-bg')
         self.webalbumpic_type = self.config.get('webgal', 'webalbumpic-type')
@@ -454,7 +477,7 @@ class WebalbumDir(make.FileMakeObject):
 
     def get_webalbumpic_filename(self):
         if self.webalbumpic_bg == 'transparent':
-            ext = '.png' # JPEG does not have an alpha channel
+            ext = '.png'  # JPEG does not have an alpha channel
         else:
             ext = '.jpg'
         return genmedia.WebalbumPicture.BASEFILENAME + ext
@@ -468,8 +491,8 @@ class WebalbumDir(make.FileMakeObject):
             # Do not append default size name to HTML page filename
             return path
         elif size_name in self.browse_sizes\
-        and self.newsizers[size_name] == 'original'\
-        and extension != '.html':
+                and self.newsizers[size_name] == 'original'\
+                and extension != '.html':
             # Do not append size_name to unresized images.
             return path
         else:
@@ -502,13 +525,13 @@ class WebalbumDir(make.FileMakeObject):
             len(self.source_dir.subdirs)
 
     def get_all_subgals(self):
-        all_subgals = list(self.subgals) # We want a copy here.
+        all_subgals = list(self.subgals)  # We want a copy here.
         for subgal in self.subgals:
             all_subgals.extend(subgal.get_all_subgals())
         return all_subgals
 
     def get_all_medias_tasks(self):
-        all_medias = list(self.medias) # We want a copy here.
+        all_medias = list(self.medias)  # We want a copy here.
         for subgal in self.subgals:
             all_medias.extend(subgal.get_all_medias_tasks())
         return all_medias
@@ -516,7 +539,7 @@ class WebalbumDir(make.FileMakeObject):
     def should_be_flattened(self, path=None):
         if path is None: path = self.source_dir.path
         return self.album.dir_flattening_depth is not False\
-        and self.source_dir.get_album_level(path) > self.album.dir_flattening_depth
+            and self.source_dir.get_album_level(path) > self.album.dir_flattening_depth
 
     def flatten_below(self):
         if self.album.dir_flattening_depth is False:
@@ -574,8 +597,8 @@ class WebalbumDir(make.FileMakeObject):
                 # FIXME: No clue why this happens, but it happens!
                 dest_file = dest_file.decode(sys.getfilesystemencoding())
             if dest_file not in self.output_items and\
-               dest_file not in expected_dirs and\
-               dest_file not in extra_files:
+                dest_file not in expected_dirs and\
+                    dest_file not in extra_files:
                 foreign_files.append(dest_file)
 
         return foreign_files
@@ -596,6 +619,10 @@ class WebalbumDir(make.FileMakeObject):
         # update here if something has been built.
         if needed_build: os.utime(self.path, None)
 
+    def media_done(self):
+        if self.progress is not None:
+            self.progress.media_done()
+
 
 class SharedFiles(make.FileMakeObject):
 
@@ -607,22 +634,25 @@ class SharedFiles(make.FileMakeObject):
         if not os.path.isdir(self.path):
             logging.info(_("MKDIR %SHAREDDIR%"))
             logging.debug("(%s)" % self.path)
-            os.makedirs(self.path, mode = 0755)
+            os.makedirs(self.path, mode=0755)
 
         super(SharedFiles, self).__init__(self.path)
 
         self.expected_shared_files = []
-        for shared_file in glob.glob(\
-          os.path.join(self.album.tpl_dir, THEME_SHARED_FILE_PREFIX + '*')):
+        for shared_file in glob.glob(
+                os.path.join(self.album.theme.tpl_dir,
+                             tpl.THEME_SHARED_FILE_PREFIX + '*')):
             shared_file_name = os.path.basename(shared_file).\
-                                     replace(THEME_SHARED_FILE_PREFIX, '')
+                replace(tpl.THEME_SHARED_FILE_PREFIX, '')
             shared_file_dest = os.path.join(self.path,
                                             shared_file_name)
 
-            if self.album.tpl_loader.is_known_template_type(shared_file):
+            if self.album.theme.tpl_loader.is_known_template_type(shared_file):
                 sf = genpage.SharedFileTemplate(album, shared_file,
                                                 shared_file_dest,
                                                 tpl_vars)
+                if self.album.force_gen_pages:
+                    sf.stamp_delete()
                 self.expected_shared_files.append(sf.path)
             else:
                 sf = genfile.SharedFileCopy(shared_file, shared_file_dest)
@@ -638,7 +668,36 @@ class SharedFiles(make.FileMakeObject):
                 self.album.cleanup(file_path)
 
 
-class Album:
+class AlbumGenProgress(object):
+
+    def __init__(self, dirs_total, medias_total):
+        self._dirs_total = dirs_total
+        self._dirs_done = 0
+
+        self._medias_total = medias_total
+        self._medias_done = 0
+
+    def dir_done(self):
+        self._dirs_done = self._dirs_done + 1
+        self.updated()
+
+    def media_done(self, how_many=1):
+        self._medias_done = self._medias_done + how_many
+        self.updated()
+
+    def __unicode__(self):
+        return _("Progress: dir %d/%d (%d%%), media %d/%d (%d%%)")\
+               % (self._dirs_done, self._dirs_total,
+                  100 * self._dirs_done // self._dirs_total,
+                  self._medias_done, self._medias_total,
+                  100 * self._medias_done // self._medias_total,
+                 )
+
+    def updated(self):
+        pass
+
+
+class Album(object):
 
     def __init__(self, source_dir, config=None):
         self.source_dir = os.path.abspath(source_dir)
@@ -656,7 +715,7 @@ class Album:
             except LazygalConfigDeprecated:
                 logging.error(_("'%s' uses a deprecated syntax: please refer to lazygal.conf(5) manual page.") % sourcedir_configfile)
                 sys.exit(1)
-        if config is not None: # Supplied config
+        if config is not None:  # Supplied config
             self.config.load(config)
 
         if self.config.getboolean('runtime', 'quiet'):
@@ -667,52 +726,14 @@ class Album:
         self.clean_dest = self.config.getboolean('global', 'clean-destination')
         self.force_gen_pages = self.config.getboolean('global', 'force-gen-pages')
 
-        self.tpl_loader = None
-
         self.set_theme(self.config.get('global', 'theme'))
 
         self.dir_flattening_depth = self.config.getint('global', 'dir-flattening-depth')
 
-    def set_theme(self, theme=tpl.DEFAULT_TEMPLATE):
-        self.theme = theme
+        self.__statistics = None
 
-        # First try user directory
-        self.tpl_dir = os.path.join(USER_THEME_DIR, self.theme)
-        if not os.path.exists(self.tpl_dir):
-            # Fallback to system themes
-            self.tpl_dir = os.path.join(THEME_DIR, self.theme)
-            if not os.path.exists(self.tpl_dir):
-                raise ValueError(_('Theme %s not found') % self.theme)
-
-        self.tpl_loader = tpl.TplFactory(os.path.join(THEME_DIR,
-                                                      tpl.DEFAULT_TEMPLATE),
-                                         self.tpl_dir)
-
-        # Load styles templates
-        for style in self.get_avail_styles(theme):
-            style_filename = style['filename']
-            try:
-                self.tpl_loader.load(style_filename)
-            except ValueError:
-                # Not a known emplate ext, ignore
-                pass
-
-    def get_avail_styles(self, theme, default_style=None):
-        style_files_mask = os.path.join(self.tpl_dir,
-                                        THEME_SHARED_FILE_PREFIX + '*' + 'css')
-        styles = []
-        for style_tpl_file in glob.glob(style_files_mask):
-            style = {}
-            tpl_filename = os.path.basename(style_tpl_file).split('.')[0]
-            style['filename'] = tpl_filename[len(THEME_SHARED_FILE_PREFIX):]
-            style['name'] = self._str_humanize(style['filename'])
-            if default_style is not None:
-                if style['filename'] == default_style:
-                    style['rel'] = 'stylesheet'
-                else:
-                    style['rel'] = 'alternate stylesheet'
-            styles.append(style)
-        return styles
+    def set_theme(self, theme=tpl.DEFAULT_THEME):
+        self.theme = tpl.Theme(os.path.join(DATAPATH, 'themes'), theme)
 
     def _str_humanize(self, text):
         dash_replaced = text.replace('_', ' ')
@@ -730,50 +751,38 @@ class Album:
             text = _('you should ')
         logging.info(_('  %sRM %s') % (text, file_path))
 
-    def walk(self, top, walked=None):
-        """
-        This is a wrapper around os.walk() from the standard library:
-        - browsing with topdown=False
-        - following symbolic links on directories
-        - whith barriers in place against walking twice the same directory,
-          which may happen when two directory trees have symbolic links to
-          each other's contents.
-        """
-        if walked is None: walked = []
-
-        for root, dirs, files in os.walk(top, topdown=False):
-            walked.append(os.path.realpath(root))
-
-            # Follow symlinks if they have not been walked yet
-            for d in dirs:
-                d_path = os.path.join(root, d)
-                if os.path.islink(d_path):
-                    if os.path.realpath(d_path) not in walked:
-                        for x in self.walk(d_path, walked):
-                            yield x
-                    else:
-                        logging.warning(_("Not following symlink '%s' because directory has already been processed.") % d_path)
-
-            yield root, dirs, files
-
     def generate_default_metadata(self):
         """
-        Generate default metada files if no exists.
+        Generate default metadata files if no exists.
         """
         logging.debug(_("Generating metadata in %s") % self.source_dir)
 
-        for root, dirnames, filenames in self.walk(self.source_dir):
-            filenames.sort() # This is required for the ignored files
-                             # checks to be reliable.
+        for root, dirnames, filenames in pathutils.walk(self.source_dir):
+            filenames.sort()  # This is required for the ignored files
+                              # checks to be reliable.
             source_dir = sourcetree.Directory(root, [], filenames, self)
             logging.info(_("[Entering %%ALBUMROOT%%/%s]") % source_dir.strip_root())
             logging.debug("(%s)" % source_dir.path)
 
             metadata.DefaultMetadata(source_dir, self).make()
 
-    def generate(self, dest_dir=None):
+    def stats(self):
+        if self.__statistics is None:
+            self.__statistics = {
+                'total' : 0,
+                'bydir' : {}
+            }
+            for root, dirnames, filenames in pathutils.walk(self.source_dir):
+                dir_medias = len([f for f in filenames\
+                                  if sourcetree.MediaHandler.is_known_media(f)])
+                self.__statistics['total'] = self.__statistics['total']\
+                                             + dir_medias
+                self.__statistics['bydir'][root] = dir_medias
+        return self.__statistics
+
+    def generate(self, dest_dir=None, progress=None):
         if dest_dir is None:
-            dest_dir = self.config.getstr('global', 'destdir')
+            dest_dir = self.config.getstr('global', 'output-directory')
         else:
             dest_dir = dest_dir.decode(sys.getfilesystemencoding())
         sane_dest_dir = os.path.abspath(os.path.expanduser(dest_dir))
@@ -792,11 +801,11 @@ class Album:
             feed = None
 
         dir_heap = {}
-        for root, dirnames, filenames in self.walk(self.source_dir):
+        for root, dirnames, filenames in pathutils.walk(self.source_dir):
 
             if root in dir_heap:
                 subdirs, subgals = dir_heap[root]
-                del dir_heap[root] # No need to keep it there
+                del dir_heap[root]  # No need to keep it there
             else:
                 subdirs = []
                 subgals = []
@@ -816,13 +825,18 @@ class Album:
 
             source_dir = sourcetree.Directory(root, subdirs, filenames, self)
 
+            destgal = WebalbumDir(source_dir, subgals, self, sane_dest_dir,
+                                  progress)
+
+            if source_dir.is_album_root():
+                # Use root config tpl vars for shared files
+                tpl_vars = destgal.tpl_vars
+
             if source_dir.get_all_medias_count() < 1:
                 logging.debug(_("(%s) and childs have no known medias, skipped")
                               % source_dir.path)
                 continue
 
-            destgal = WebalbumDir(source_dir, subgals, self, sane_dest_dir)
-
             if not source_dir.is_album_root():
                 container_dirname = os.path.dirname(root)
                 if container_dirname not in dir_heap:
@@ -830,9 +844,6 @@ class Album:
                 container_subdirs, container_subgals = dir_heap[container_dirname]
                 container_subdirs.append(source_dir)
                 container_subgals.append(destgal)
-            else:
-                # Use root config tpl vars for shared files
-                tpl_vars = destgal.tpl_vars
 
             if feed and source_dir.is_album_root():
                 feed.set_title(source_dir.human_name)
@@ -848,14 +859,19 @@ class Album:
             if check_all_dirs:
                 destgal.make()
             elif destgal.needs_build():
-                destgal.make(force=True) # avoid another needs_build() call in make()
+                destgal.make(force=True)  # avoid another needs_build() call in make()
             else:
+                if progress is not None:
+                    progress.media_done(len(destgal.medias))
                 logging.debug(_("  SKIPPED because of mtime, touch source or use --check-all-dirs to override."))
 
             # Force some memory cleanups, this is usefull for big albums.
             del destgal
             gc.collect()
 
+            if progress is not None:
+                progress.dir_done()
+
             logging.info(_("[Leaving  %%ALBUMROOT%%/%s]") % source_dir.strip_root())
 
         if feed:
