@@ -19,16 +19,16 @@
 import os
 import logging
 
-import Image as PILImage
+from PIL import Image as PILImage
 # lazygal has her own ImageFile class, so avoid trouble
-import ImageFile as PILImageFile
-PILImageFile.MAXBLOCK = 1024*1024 # default is 64k, not enough for big pics
+from PIL import ImageFile as PILImageFile
+PILImageFile.MAXBLOCK = 1024 * 1024  # default is 64k, not enough for big pics
 
 import make
 import genfile
 import eyecandy
 import mediautils
-from lazygal import pyexiv2api as pyexiv2
+from lazygal.pygexiv2 import GExiv2
 
 
 THUMB_SIZE_NAME = 'thumb'
@@ -77,7 +77,7 @@ class ResizedImage(genfile.WebalbumFile):
             self.build()
         except IOError:
             if not self.source_media.broken:
-                logging.error(_("  %s is BROKEN, skipped")\
+                logging.error(_("  %s is BROKEN, skipped")
                               % self.source_media.filename)
                 self.source_media.broken = True
 
@@ -87,7 +87,7 @@ class ResizedImage(genfile.WebalbumFile):
             self.stamp_build()
 
     def resize(self, im):
-        self.source_media.get_size() # Probe brokenness
+        self.source_media.get_size()  # Probe brokenness
         if self.source_media.broken:
             raise IOError()
 
@@ -101,7 +101,7 @@ class ResizedImage(genfile.WebalbumFile):
         while not calibrated:
             try:
                 im.save(self.path, quality=self.webgal.quality,
-                                   **self.webgal.save_options)
+                        **self.webgal.save_options)
             except IOError, e:
                 if str(e).startswith('encoder error'):
                     PILImageFile.MAXBLOCK = 2 * PILImageFile.MAXBLOCK
@@ -149,13 +149,13 @@ class ImageOtherSize(ResizedImage):
         return PILImage.open(self.source_media.path)
 
     TRANSPOSE_METHODS = {
-        90  : PILImage.ROTATE_90,
-        180 : PILImage.ROTATE_180,
-        270 : PILImage.ROTATE_270,
+        90 : PILImage.ROTATE_90,
+        180: PILImage.ROTATE_180,
+        270: PILImage.ROTATE_270,
     }
 
     def resize(self, im):
-        self.source_media.get_size() # Probe brokenness
+        self.source_media.get_size()  # Probe brokenness
         if self.source_media.broken:
             raise IOError()
 
@@ -179,20 +179,19 @@ class ImageOtherSize(ResizedImage):
     )
 
     def copy_metadata(self):
-        imgtags = pyexiv2.ImageMetadata(self.source_media.path)
-        imgtags.read()
-        dest_imgtags = pyexiv2.ImageMetadata(self.path)
-        dest_imgtags.read()
-        imgtags.copy(dest_imgtags)
+        imgtags = GExiv2.Metadata(self.source_media.path)
+        dest_imgtags = GExiv2.Metadata(self.path)
+        for tag in imgtags.get_exif_tags():
+            dest_imgtags[tag] = imgtags[tag]
 
         new_size = self.get_size()
-        dest_imgtags['Exif.Photo.PixelXDimension'] = new_size[0]
-        dest_imgtags['Exif.Photo.PixelYDimension'] = new_size[1]
+        dest_imgtags['Exif.Photo.PixelXDimension'] = str(new_size[0])
+        dest_imgtags['Exif.Photo.PixelYDimension'] = str(new_size[1])
 
         if self.get_rotation() != 0:
             # Smaller image has been rotated in order to be displayed correctly
             # in a web browser. Fix orientation tag accordingly.
-            dest_imgtags['Exif.Image.Orientation'] = 1
+            dest_imgtags['Exif.Image.Orientation'] = '1'
 
         # Those are removed from published pics due to pivacy concerns,
         # unless explicitly told to keep them in. Option to retain GPS
@@ -200,12 +199,12 @@ class ImageOtherSize(ResizedImage):
         if not self.webgal.keep_gps:
             for tag in self.PRIVATE_IMAGE_TAGS:
                 try:
-                    del dest_imgtags[tag]
+                    dest_imgtags.clear_tag(tag)
                 except KeyError:
                     pass
         try:
-            dest_imgtags.write()
-        except ValueError, e:
+            dest_imgtags.save_file()
+        except Exception, e:
             logging.error(_("Could not copy metadata in reduced picture: %s") % e)
 
     def save(self, im):
@@ -226,7 +225,7 @@ class VideoThumb(ResizedImage):
         try:
             thumb = mediautils.VideoThumbnailer(self.source_media.path).get_thumb()
         except mediautils.TranscodeError, e:
-            logging.error(_("  creating %s thumbnail failed, skipped")\
+            logging.error(_("  creating %s thumbnail failed, skipped")
                           % self.source_media.filename)
             logging.info(str(e))
             self.clean_output()
@@ -248,7 +247,7 @@ class WebalbumPicture(make.FileMakeObject):
 
         # Use already generated thumbs for better performance (lighter to
         # rotate, etc.).
-        thumbs = [image.thumb\
+        thumbs = [image.thumb
                   for image in webgal_dir.get_all_medias_tasks()
                   if image.thumb and not image.media.broken]
 
@@ -259,12 +258,11 @@ class WebalbumPicture(make.FileMakeObject):
             albumpic_path = os.path.join(webgal_dir.source_dir.path,
                                          webgal_dir.source_dir.album_picture)
             if not os.path.isfile(albumpic_path):
-                logging.error(_("Supplied album picture %s does not exist.")\
+                logging.error(_("Supplied album picture %s does not exist.")
                               % webgal_dir.source_dir.album_picture)
 
-            md_dirpic_thumb = webgal_dir._add_size_qualifier(\
-                                           webgal_dir.source_dir.album_picture,
-                                           THUMB_SIZE_NAME)
+            md_dirpic_thumb = webgal_dir._add_size_qualifier(
+                webgal_dir.source_dir.album_picture, THUMB_SIZE_NAME)
             md_dirpic_thumb = os.path.join(webgal_dir.path, md_dirpic_thumb)
         else:
             md_dirpic_thumb = None
@@ -310,7 +308,7 @@ class WebVideo(genfile.WebalbumFile):
             width, height = self.newsizer.dest_size(self.source_video.get_size())
             mediautils.WebMTranscoder(self.source_video.path, width, height).convert(self.path)
         except mediautils.TranscodeError, e:
-            logging.error(_("  transcoding %s failed, skipped")\
+            logging.error(_("  transcoding %s failed, skipped")
                           % self.source_video.filename)
             logging.info(str(e))
             self.source_video.broken = True
