@@ -39,6 +39,8 @@ else:
 
 
 interrupted = False
+
+
 def signal_handler(signum, frame):
     global interrupted
     interrupted = True
@@ -51,7 +53,7 @@ def gobject_init():
     signal.signal(signal.SIGINT, signal_handler)
 
 
-import Image as PILImage
+from PIL import Image as PILImage
 
 
 class TranscodeError(Exception): pass
@@ -95,8 +97,8 @@ class GstVideoOpener(object):
         if not gobjects_threads_init:
             gobject_init()
 
-        self.filesrc.set_property("location",
-                            self.input_file.encode(sys.getfilesystemencoding()))
+        self.filesrc.set_property(
+            "location", self.input_file.encode(sys.getfilesystemencoding()))
 
     def check_interrupt(self):
         if interrupted:
@@ -104,7 +106,7 @@ class GstVideoOpener(object):
                                               gst.Structure('interrupted'))
             self.pipeline.get_bus().post(msg)
         if interrupted or not self.running:
-            return False # Remove timeout handler
+            return False  # Remove timeout handler
         return True
 
     def __stop_pipeline(self):
@@ -146,6 +148,23 @@ class GstVideoInfo(object):
         self.loop = gobject.MainLoop()
 
     def __stream_discovered(self, obj, success):
+        if success:
+            self.videowidth = obj.videowidth
+            self.videoheight = obj.videoheight
+            self.videorate = obj.videorate
+            self.audiorate = obj.audiorate
+            self.audiodepth = obj.audiodepth
+            self.audiowidth = obj.audiowidth
+            self.audiochannels = obj.audiochannels
+
+            self.audiolength = obj.audiolength
+            self.videolength = obj.videolength
+
+            self.is_video = obj.is_video
+            self.is_audio = obj.is_audio
+        else: # TODO : handle error
+            pass
+
         self.loop.quit()
 
     def inspect(self):
@@ -158,21 +177,6 @@ class GstVideoInfo(object):
         discoverer.discover()
         self.loop.run()
 
-        self.videowidth = discoverer.videowidth
-        self.videoheight = discoverer.videoheight
-        self.videorate = discoverer.videorate
-        self.audiorate = discoverer.audiorate
-        self.audiodepth = discoverer.audiodepth
-        self.audiowidth = discoverer.audiowidth
-        self.audiochannels = discoverer.audiochannels
-
-        self.audiolength = discoverer.audiolength
-        self.videolength = discoverer.videolength
-
-        self.is_video = discoverer.is_video
-        self.is_audio = discoverer.is_audio
-
-
 class GstVideoReader(GstVideoOpener):
 
     def __init__(self, mediapath):
@@ -344,8 +348,8 @@ class VideoFrameExtractor(GstVideoReader):
 
         # RGB is what is assumed in order to load the frame into a PIL Image.
         self.capsfilter = gst.element_factory_make('capsfilter')
-        self.capsfilter.set_property('caps',
-                              gst.Caps('video/x-raw-rgb,framerate=%s/1' % fps))
+        self.capsfilter.set_property(
+            'caps', gst.Caps('video/x-raw-rgb,framerate=%s/1' % fps))
         self.pipeline.add(self.capsfilter)
         videorate.link(self.capsfilter)
 
@@ -402,7 +406,7 @@ class VideoBestFrameFinder(VideoFrameExtractor):
     def __init__(self, path, fps, intro_seconds):
         super(VideoBestFrameFinder, self).__init__(path, fps)
 
-        self.max_frames = self.fps * intro_seconds # Frames to go through
+        self.max_frames = self.fps * intro_seconds  # Frames to go through
         self.histograms = []
 
     def cb_grabbed_frame_buf(self, buf):
@@ -427,7 +431,7 @@ class VideoBestFrameFinder(VideoFrameExtractor):
         for value_index in range(n_values):
             average = 0.0
             for histogram in self.histograms:
-                average = average + (float(histogram[value_index])/n_samples)
+                average = average + (float(histogram[value_index]) / n_samples)
             average_hist.append(average)
 
         # Find histogram closest to average histogram
@@ -438,7 +442,7 @@ class VideoBestFrameFinder(VideoFrameExtractor):
             mse = 0.0
             for value_index in range(n_values):
                 gap = average_hist[value_index] - hist[value_index]
-                mse = mse + gap*gap
+                mse = mse + gap * gap
 
             if min_mse is None or mse < min_mse:
                 min_mse = mse
@@ -476,7 +480,8 @@ class VideoThumbnailer(object):
 
 
 if __name__ == '__main__':
-    import sys, os
+    import sys
+    import os
 
     converter_types = sys.argv[1].split(',')
     converters = {}
@@ -497,7 +502,18 @@ if __name__ == '__main__':
         file_path = file_path.decode(sys.getfilesystemencoding())
         fn, ext = os.path.splitext(os.path.basename(file_path))
         for converter_type, converter in converters.items():
-            target_path = fn + '.' + converter_type
+
+            counter_str = ''
+            counter = 0
+            filename_free = False
+            while not filename_free:
+                target_path = fn + counter_str + '.' + converter_type
+                if os.path.isfile(target_path):
+                    counter = counter + 1
+                    counter_str = '_%d' % counter
+                else:
+                    filename_free = True
+
             converter(file_path).convert(target_path)
 
 
