@@ -15,14 +15,20 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import os, locale
+from __future__ import division
+
+import os
+import locale
 import logging
 import codecs
 import datetime
 
-from lazygal import pyexiv2api as pyexiv2
+from lazygal.pygexiv2 import GExiv2
+from PIL import Image as PILImage
+
 from lazygal import make
 
+from fractions import Fraction
 
 FILE_METADATA_ENCODING = locale.getpreferredencoding()
 
@@ -37,6 +43,70 @@ MATEW_METADATA = 'album_description'
 FILE_METADATA = ('album-name', 'album-description', 'album-picture', )
 FILE_METADATA_MEDIA_SUFFIX = '.comment'
 
+# As per http://www.exiv2.org/tags.html
+VENDOR_EXIF_CODES = (
+    'Exif.Canon.LensModel',
+    'Exif.Minolta.LensID',
+    'Exif.Nikon3.Lens',
+    'Exif.Nikon3.LensType',
+    'Exif.OlympusEq.LensModel',
+    'Exif.OlympusEq.LensType',
+    'Exif.Panasonic.LensType',
+    'Exif.Pentax.LensType',
+    'Exif.Samsung2.LensType',
+    'Exif.Sigma.LensRange',
+    'Exif.Sony1.LensID',
+    )
+
+GEXIV2_DATE_FORMAT = '%Y:%m:%d %H:%M:%S'
+GExiv2.log_set_level(GExiv2.LogLevel.ERROR) # hide exiv2 warnings
+
+
+def decode_exif_user_comment(raw, imgpath):
+    """
+    GExiv2 does not decode EXIF user comment.
+    """
+    # This field can contain charset information
+    if raw.startswith('charset='):
+        tokens = raw.split(' ')
+        csetfield = tokens[0]
+        text = ' '.join(tokens[1:])
+        ignore, cset = csetfield.split('=')
+        cset = cset.strip('"')
+    else:
+        cset = None
+        text = raw
+
+    if cset == 'Unicode':
+        encoding = None
+        try:
+            text.decode('utf-8')
+        except UnicodeDecodeError:
+            im = PILImage.open(imgpath)
+            endianess = im.app['APP1'][6:8]
+            if endianess == 'MM':
+                encoding = 'utf-16be'
+            elif endianess == 'II':
+                encoding = 'utf-16le'
+            else:
+                raise ValueError
+        else:
+            encoding = 'utf-8'
+    elif cset == 'Ascii':
+        encoding = 'ascii'
+    elif cset == 'Jis':
+        encoding = 'shift_jis'
+    else:
+        # Fallback to utf-8 as this is mostly the default for Linux
+        # distributions.
+        encoding = 'utf-8'
+
+    # Return the decoded string according to the found encoding.
+    try:
+        return text.decode(encoding)
+    except UnicodeDecodeError:
+        return text.decode(encoding, 'replace')
+
 
 class FileMetadata(object):
 
@@ -68,51 +138,38 @@ class ImageInfoTags(object):
 
     def __init__(self, image_path):
         self.image_path = image_path
-        self._metadata = pyexiv2.ImageMetadata(self.image_path)
-        self._metadata.read()
-
-    def get_tag_value(self, name):
-        if name.startswith('Iptc.'):
-            # Iptc tags are always lists, so, for now, return the first
-            # element.
-            return self._metadata[name].values[0]
-        else:
-            return self._metadata[name].value
+        self._metadata = GExiv2.Metadata(self.image_path)
 
     def get_date(self):
         """
         Get real time when photo has been taken. We prefer EXIF fields
-        as those were filled by camera, Image DateTime can be update by
+        as those were filled by camera, Image DateTime can be updated by
         software when editing photos later.
         """
-        for date_tag in ('Exif.Photo.DateTimeDigitized',
-                         'Exif.Photo.DateTimeOriginal',
-                         'Exif.Image.DateTime',
-                        ):
+        for tag in ('Exif.Photo.DateTimeOriginal',
+                    'Exif.Image.DateTimeDigitized',
+                    'Exif.Image.DateTime',
+                   ):
             try:
-                date = self.get_tag_value(date_tag)
-                if type(date) is not datetime.datetime:
-                    # Sometimes, pyexiv2 sends a string. It seems to happen on
-                    # malformed tags.
-                    raise ValueError
-            except (IndexError, ValueError, KeyError):
+                dt_str = self._metadata[tag]
+                dt = datetime.datetime.strptime(dt_str, GEXIV2_DATE_FORMAT)
+            except (KeyError, ValueError) as StrptimeError:
+                # ValueError: bypass errors such as "time data '0000:00:00
+                # 00:00:00' does not match format '%Y:%m:%d %H:%M:%S'"
                 pass
             else:
-                return date
-
-        # No date could be found in the picture metadata
-        return None
+                return dt
 
     def get_required_rotation(self):
         try:
-            orientation_code = self.get_tag_value('Exif.Image.Orientation')
+            orientation_code = int(self._metadata['Exif.Image.Orientation'])
             if orientation_code == 8:
                 return 90
             elif orientation_code == 3:
                 return 180
             elif orientation_code == 6:
                 return 270
-            else: # Should be orientation_code == 1 but catch all
+            else:  # Should be orientation_code == 1 but catch all
                 return 0
         except KeyError:
             return 0
@@ -124,13 +181,13 @@ class ImageInfoTags(object):
         vendors put different information to both tags.
         """
         try:
-            model = self.get_tag_value('Exif.Image.Model').strip()
+            model = self._metadata['Exif.Image.Model'].strip()
             # Terminate string at \x00
             pos = model.find('\x00')
             if pos != -1:
                 model = model[:14]
             try:
-                vendor = self.get_tag_value('Exif.Image.Make').strip()
+                vendor = self._metadata['Exif.Image.Make'].strip()
                 vendor_l = vendor.lower()
                 model_l = model.lower()
                 # Split vendor to words and check whether they are
@@ -141,7 +198,7 @@ class ImageInfoTags(object):
                 for word in vendor_l.split(' '):
                     if model_l.find(word) != -1:
                         return model
-                return '%s %s' % (vendor, model)
+                return ' '.join([vendor, model])
             except KeyError:
                 return model
         except KeyError:
@@ -150,33 +207,26 @@ class ImageInfoTags(object):
     def get_lens_name(self):
         """
         Return name of used lenses. This usually makes sense only for
-        SLR cameras and uses various maker notes. Currently supported
-        for Pentax, Nikon and Minolta (as soon as Exiv2 supports others,
-        support can be added here.
+        SLR cameras and uses various maker notes.
         """
-
-        try:
-            return self._metadata['Exif.Pentax.LensType'].human_value.strip()
-        except (IndexError, ValueError, KeyError):
+        interpret = self._metadata.get_tag_interpreted_string
+        vendor_values = []
+        for key in VENDOR_EXIF_CODES:
             try:
-                ret = self._metadata['Exif.Nikon3.Lens'].human_value.strip()
-                try:
-                    ret2 = self._metadata['Exif.Nikon3.LensType'].human_value.strip()
-                except (IndexError, ValueError, KeyError):
-                    return ret
-                else:
-                    return '%s %s' % (ret, ret2)
-            except (IndexError, ValueError, KeyError):
-                try:
-                    return self._metadata['Exif.Minolta.LensID'].human_value.strip()
-                except (IndexError, ValueError, KeyError):
-                    return ''
+                v = self._metadata.get_tag_interpreted_string(key)
+                if v is None:
+                    raise KeyError
+            except KeyError:
+                pass
+            else:
+                vendor_values.append(v.strip())
+        return ' '.join([s for s in vendor_values if s])
 
     def get_exif_string(self, name):
         """
         Reads string from EXIF information.
         """
-        return str(self.get_tag_value(name)).strip(' ')
+        return self._metadata[name].strip(' ')
 
     def get_exif_float(self, name):
         """
@@ -191,15 +241,11 @@ class ImageInfoTags(object):
         Reads float number from EXIF information (where it is stored as
         fraction or int).
         """
-        val = self.get_tag_value(name)
-        if type(val) == int:
-            return float(val)
-        elif type(val) == tuple:
-            return float(val[0]) / float(val[1])
-        else:
-            return float(val.numerator) / float(val.denominator)
+        val = self._metadata.get_exif_tag_rational(name)
+        return val.numerator / val.denominator
 
     def _fallback_to_encoding(self, encoded_string, encoding='utf-8'):
+        if encoded_string is None: raise ValueError
         if type(encoded_string) is unicode: return encoded_string
         try:
             return encoded_string.decode(encoding)
@@ -207,7 +253,9 @@ class ImageInfoTags(object):
             return encoded_string.decode(encoding, 'replace')
 
     def get_exif_usercomment(self):
-        ret = self.get_tag_value('Exif.Photo.UserComment').strip(' \0\x00')
+        ret = self._metadata['Exif.Photo.UserComment'].strip(' \0\x00')
+        if type(ret) is not unicode: # the EXIF lib did not do the work for us
+            ret = decode_exif_user_comment(ret, self.image_path)
         if ret == 'User comments':
             return ''
         return ret
@@ -221,16 +269,15 @@ class ImageInfoTags(object):
             ret = self.get_file_comment()
             if ret is None:
                 ret = self.get_exif_usercomment()
-                ret = self._fallback_to_encoding(ret)
                 if ret == '':
                     raise ValueError
         except (ValueError, KeyError):
             try:
-                ret = self.get_exif_string('Exif.Image.ImageDescription')
+                ret = self._metadata['Exif.Image.ImageDescription']
                 ret = self._fallback_to_encoding(ret)
             except (ValueError, KeyError):
                 try:
-                    ret = self.get_exif_string('Iptc.Application2.ObjectName')
+                    ret = self._metadata['Iptc.Application2.ObjectName']
                     ret = self._fallback_to_encoding(ret)
                 except (ValueError, KeyError):
                     ret = self.get_jpeg_comment()
@@ -238,50 +285,42 @@ class ImageInfoTags(object):
 
     def get_flash(self):
         try:
-            return self._metadata['Exif.Photo.Flash'].human_value
-        except KeyError:
+            flash_info = self._metadata.get_tag_interpreted_string('Exif.Photo.Flash')
+            return self._fallback_to_encoding(flash_info)
+        except (ValueError, KeyError):
             return ''
 
     def get_exposure(self):
         try:
-            exposure = self.get_tag_value('Exif.Photo.ExposureTime')
-            if type(exposure) == tuple:
-                if exposure[1] == 1:
-                    return "%d s" % exposure[0]
-                else:
-                    return "%d/%d s" % (exposure[0], exposure[1])
-            else:
-                if exposure.denominator == 1:
-                    return "%d s" % exposure.numerator
-                else:
-                    return "%d/%d s" % (exposure.numerator, exposure.denominator)
+            return str(
+                self._metadata.get_exif_tag_rational('Exif.Photo.ExposureTime'))
         except (ValueError, KeyError):
             return ''
 
     def get_iso(self):
         try:
-            return self.get_exif_string('Exif.Photo.ISOSpeedRatings')
+            return self._metadata['Exif.Photo.ISOSpeedRatings']
         except KeyError:
             return ''
 
     def get_fnumber(self):
         try:
-            val = self.get_exif_float('Exif.Photo.FNumber')
-        except KeyError:
+            val = float(self._metadata.get_exif_tag_rational('Exif.Photo.FNumber'))
+        except (KeyError, TypeError):
             return ''
         else:
-            return 'f/%s' % val
+            return 'f/{}'.format(val)
 
     def get_focal_length(self):
         try:
-            flen = self.get_exif_float('Exif.Photo.FocalLength')
+            flen = self._metadata.get_exif_tag_rational('Exif.Photo.FocalLength')
         except KeyError:
             return ''
         else:
             flen = '%s mm' % flen
 
         try:
-            flen35 = self.get_exif_float('Exif.Photo.FocalLengthIn35mmFilm')
+            flen35 = self._metadata.get_exif_tag_rational('Exif.Photo.FocalLengthIn35mmFilm')
         except KeyError:
             pass
         else:
@@ -294,7 +333,7 @@ class ImageInfoTags(object):
             except (IndexError, KeyError):
                 iwidth = self.get_exif_float_value('Exif.Photo.PixelXDimension')
 
-            fresunit = str(self._metadata['Exif.Photo.FocalPlaneResolutionUnit'].value)
+            fresunit = self._metadata['Exif.Photo.FocalPlaneResolutionUnit']
             factors = {'1': 25.4, '2': 25.4, '3': 10, '4': 1, '5': 0.001}
             try:
                 fresfactor = factors[fresunit]
@@ -321,19 +360,51 @@ class ImageInfoTags(object):
 
     def get_jpeg_comment(self):
         try:
-            comment = self._metadata.comment.strip(' ')
-            if '\x00' in comment:
-                raise ValueError # ignore broken JPEG comments
-            return self._fallback_to_encoding(comment)
-        except (AttributeError, ValueError):
+            comment = self._metadata.get_comment()
+            if comment is None or '\x00' in comment:
+                raise ValueError  # ignore missing or broken JPEG comments
+            return self._fallback_to_encoding(comment.strip(' '))
+        except ValueError:
             return ''
 
     def get_authorship(self):
         try:
-            return self.get_exif_string('Exif.Image.Artist')
+            author = self._metadata['Exif.Image.Artist']
+            return self._fallback_to_encoding(author)
         except KeyError:
             return ''
 
+    def get_keywords(self):
+        """
+        Returns all the image tags in a list.
+
+        Try to find the maximum number of keywords. Photo applications store
+        keywords in various places. For a comprehensive list, see
+        http://redmine.yorba.org/projects/shotwell/wiki/PhotoTags
+        """
+        kw = list()
+        for key in ('Iptc.Application2.Keywords',
+                    'Xmp.MicrosoftPhoto.LastKeywordXMP',
+                    'Xmp.dc.subject',
+                    'Xmp.digiKam.TagsList', ):
+            try:
+                values = self._metadata.get_tag_multiple(key)
+            except KeyError:
+                pass
+            else:
+                for value in values:
+                    kw.append(value)
+        # FIXME
+        # Reading the metadata Xmp.lr.hierarchicalSubject produces error
+        # messages:
+        #   "No namespace info available for XMP prefix `lr'"
+        #kw += self._metadata.get_tag_multiple('Xmp.lr.hierarchicalSubject')
+
+        #remove duplicates
+        kw = set(kw)
+
+        return kw
+
 
 class NoMetadata(Exception):
     """
@@ -363,7 +434,7 @@ class DirectoryMetadata(make.GroupTask):
                 if os.path.isfile(file_md_path):
                     self.add_file_dependency(file_md_path)
 
-    def get_matew_metadata(self, metadata, subdir = None):
+    def get_matew_metadata(self, metadata, subdir=None):
         """
         Return dictionary with meta data parsed from Matew like format.
         """
@@ -478,7 +549,7 @@ class DefaultMetadata(make.FileMakeObject):
 
         md_data = md.get(None, self.source_dir)
         if 'album_description' in md_data.keys()\
-        or 'album_name' in md_data.keys():
+                or 'album_name' in md_data.keys():
             logging.debug(_("  SKIPPED because metadata exists."))
         elif self.source_dir.get_all_medias_count() < 1:
             logging.debug(_("  SKIPPED because directory does not contain images."))
@@ -495,10 +566,10 @@ class DefaultMetadata(make.FileMakeObject):
         f = file(self._path, 'w')
         f.write(codecs.BOM_UTF8)
         f.write('# Directory metadata for lazygal, Matew format\n')
-        f.write('Album name "%s"\n'\
+        f.write('Album name "%s"\n'
                 % self.source_dir.human_name.encode('utf-8'))
         f.write('Album description ""\n')
-        f.write('Album image identifier "%s"\n'\
+        f.write('Album image identifier "%s"\n'
                 % md['album_picture'].encode('utf-8'))
         f.close()
 
