@@ -15,10 +15,12 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import os, time
+import re
+import os
+import time
 import locale
 import logging
-import Image
+from PIL import Image
 
 from lazygal import pathutils, make, metadata
 from lazygal import mediautils, timeutils
@@ -138,7 +140,7 @@ class MediaFile(File):
         if self.has_reliable_date() and other_media.has_reliable_date():
             return self.compare_date_taken(other_media)
         elif not self.has_reliable_date()\
-        and not other_media.has_reliable_date():
+                and not other_media.has_reliable_date():
             return self.compare_filename(other_media)
         else:
             # One of the picture has no EXIF date, so we arbitrary sort it
@@ -218,20 +220,21 @@ class VideoFile(MediaFile):
 
 class MediaHandler(object):
 
-    FORMATS = { '.jpeg' : ImageFile,
-                '.jpg'  : ImageFile,
-                '.png'  : ImageFile,
-                '.mov'  : VideoFile,
-                '.avi'  : VideoFile,
-                '.mp4'  : VideoFile,
-                '.3gp'  : VideoFile,
-                '.webm' : VideoFile,
-              }
+    FORMATS = {'.jpeg': ImageFile,
+               '.jpg' : ImageFile,
+               '.png' : ImageFile,
+               '.mov' : VideoFile,
+               '.avi' : VideoFile,
+               '.mp4' : VideoFile,
+               '.3gp' : VideoFile,
+               '.webm': VideoFile,
+               }
 
     def __init__(self, album):
         self.album = album
 
-    def is_known_media(self, path):
+    @staticmethod
+    def is_known_media(path):
         filename, extension = os.path.splitext(path)
         extension = extension.lower()
         return extension in MediaHandler.FORMATS.keys()
@@ -262,6 +265,8 @@ class Directory(File):
 
         self.human_name = self.album._str_humanize(self.name)
 
+        tagfilters = album.config.getlist('webgal', 'filter-by-tag')
+
         media_handler = MediaHandler(self.album)
         self.medias = []
         self.medias_names = []
@@ -269,11 +274,34 @@ class Directory(File):
             media_path = os.path.join(self.path, filename)
             media = media_handler.get_media(media_path)
             if media:
-                self.medias_names.append(filename)
-                self.medias.append(media)
-            elif not self.is_metadata(filename)and\
-                 filename != SOURCEDIR_CONFIGFILE:
-                logging.info(_("  Ignoring %s, format not supported.")\
+
+                # tag-filtering is requested
+                if len(tagfilters) > 0:
+                    res = True
+                    for tagf in tagfilters:
+                        # concatenate the list of tags as a string of words,
+                        # space-separated.  to ensure that we match the full
+                        # keyword and not only a subpart of it, we also surround
+                        # the matching pattern with spaces
+
+                        # we look for tag words, partial matches are not wanted
+                        regex = re.compile(r"\b" + tagf + r"\b")
+
+                        kwlist = ' '.join(media.info().get_keywords())
+                        if re.search(regex, kwlist) is None:
+                            res = False
+                            break
+                    if res is True:
+                        self.medias_names.append(filename)
+                        self.medias.append(media)
+
+                # no tag-filtering  
+                else:
+                        self.medias_names.append(filename)
+                        self.medias.append(media)
+            elif not self.is_metadata(filename) and\
+                    filename != SOURCEDIR_CONFIGFILE:
+                logging.info(_("  Ignoring %s, format not supported.")
                              % filename)
                 logging.debug("(%s)" % os.path.join(self.path, filename))
 
@@ -347,7 +375,7 @@ class Directory(File):
         return all_medias_count
 
     def get_all_medias(self):
-        all_medias = list(self.medias) # We want a copy here.
+        all_medias = list(self.medias)  # We want a copy here.
         for subdir in self.subdirs:
             all_medias.extend(subdir.get_all_medias())
         return all_medias
@@ -356,7 +384,7 @@ class Directory(File):
         return [m.path for m in self.get_all_medias()]
 
     def get_all_subdirs(self):
-        all_subdirs = list(self.subdirs) # We want a copy here.
+        all_subdirs = list(self.subdirs)  # We want a copy here.
         for subdir in self.subdirs:
             all_subdirs.extend(subdir.get_all_subdirs())
         return all_subdirs
@@ -373,7 +401,7 @@ class Directory(File):
         media_stamp_max = None
         for m in all_medias:
             if m.has_reliable_date():
-                media_stamp = time.mktime(m.get_date_taken().timetuple())
+                media_stamp = m.get_date_taken().timestamp
                 if media_stamp_max is None or media_stamp > media_stamp_max:
                     media_stamp_max = media_stamp
                     if hint is not None and media_stamp_max > hint:
