@@ -21,17 +21,32 @@ import os
 import datetime
 import shutil
 
-import Image
+from PIL import Image
 
 from __init__ import LazygalTestGen, skip, has_symlinks
 import lazygal.config
 from lazygal.generators import WebalbumDir
 from lazygal.sourcetree import Directory
-from lazygal import pyexiv2api as pyexiv2
+from lazygal.metadata import GEXIV2_DATE_FORMAT
+from lazygal.pygexiv2 import GExiv2
 
 
 class TestGenerators(LazygalTestGen):
 
+    def test_albumstats(self):
+        pics = ['img%d.jpg' % i for i in range(0, 8)]
+        self.setup_subgal('subgal', pics)
+        self.setup_subgal('vidgal', ['vid.webm'])
+
+        expected_stats = {
+            self.source_dir: 0,
+            os.path.join(self.source_dir, 'subgal') : 8,
+            os.path.join(self.source_dir, 'vidgal') : 1,
+        }
+
+        self.assertEqual(self.album.stats()['total'], 9)
+        self.assertEqual(self.album.stats()['bydir'], expected_stats)
+
     def test_genfile_filelayout(self):
         source_subgal = self.setup_subgal('subgal', ['subgal_img.jpg'])
 
@@ -58,7 +73,7 @@ class TestGenerators(LazygalTestGen):
         Files that are not part of what was generated or updated shall be
         spotted.
         """
-        pics = [ 'img%d.jpg' % i for i in range(0, 8)]
+        pics = ['img%d.jpg' % i for i in range(0, 8)]
         source_subgal = self.setup_subgal('subgal', pics)
 
         dest_path = self.get_working_path()
@@ -77,14 +92,14 @@ class TestGenerators(LazygalTestGen):
         source_subgal = Directory(os.path.join(self.source_dir, 'subgal'),
                                   [], pics, self.album)
         dest_subgal = WebalbumDir(source_subgal, [], self.album, dest_path)
-        expected = map(lambda fn:\
-                             unicode(os.path.join(dest_path, 'subgal', fn)),
+        expected = map(lambda fn:
+                       unicode(os.path.join(dest_path, 'subgal', fn)),
                        ['extra_thumb2.jpg',
                         'img6_thumb.jpg',
                         'img6_small.jpg', 'img6_medium.jpg',
                         'img6.html', 'img6_medium.html',
-                       ]
-                      )
+                        ]
+                       )
         self.assertEqual(sorted(dest_subgal.list_foreign_files()),
                          sorted(expected))
 
@@ -107,7 +122,7 @@ class TestGenerators(LazygalTestGen):
 
         symlink = os.path.join(dest_dir, os.path.basename(img_path))
         # Test if the original in the webgal is a symlink
-        self.assertEqual(os.path.islink(symlink), True)
+        self.assertTrue(os.path.islink(symlink))
         # Test if that symlink point to the image in the source_dir
         self.assertEqual(os.path.realpath(symlink), img_path)
 
@@ -115,39 +130,35 @@ class TestGenerators(LazygalTestGen):
         img_path = self.add_img(self.source_dir, 'md_filled.jpg')
 
         # Add some metadata
-        gps_data = pyexiv2.ImageMetadata(self.get_sample_path('sample-with-gps.jpg'))
-        gps_data.read()
-        source_image = pyexiv2.ImageMetadata(img_path)
-        source_image.read()
-        gps_data.copy(source_image)
+        gps_data = GExiv2.Metadata(self.get_sample_path('sample-with-gps.jpg'))
+        source_image = GExiv2.Metadata(img_path)
+        for tag in gps_data.get_exif_tags():
+            source_image[tag] = gps_data[tag]
         dummy_comment = 'nice photo'
         source_image['Exif.Photo.UserComment'] = dummy_comment
         dummy_date = datetime.datetime(2011, 2, 3, 12, 51, 43)
-        source_image['Exif.Photo.DateTimeDigitized'] = dummy_date
-        assert 'Exif.GPSInfo.GPSLongitude' in source_image.exif_keys
-        assert 'Exif.GPSInfo.GPSLatitude' in source_image.exif_keys
-        source_image.write()
+        source_image['Exif.Photo.DateTimeDigitized'] = dummy_date.strftime(GEXIV2_DATE_FORMAT)
+        assert 'Exif.GPSInfo.GPSLongitude' in source_image
+        assert 'Exif.GPSInfo.GPSLatitude' in source_image
+        source_image.save_file()
 
         # Generate album
         dest_dir = self.get_working_path()
         self.album.generate(dest_dir)
 
         dest_img_path = os.path.join(dest_dir, 'md_filled_small.jpg')
-        dest_image = pyexiv2.ImageMetadata(dest_img_path)
-        dest_image.read()
+        dest_image = GExiv2.Metadata(dest_img_path)
 
         # Check that metadata is still here for reduced pictures.
-        self.assertEqual(dest_image['Exif.Photo.UserComment'].value,
-                         dummy_comment)
-        self.assertEqual(dest_image['Exif.Photo.DateTimeDigitized'].value,
-                         dummy_date)
+        self.assertEqual(dest_image['Exif.Photo.UserComment'], dummy_comment)
+        self.assertEqual(dest_image['Exif.Photo.DateTimeDigitized'], dummy_date.strftime('%Y:%m:%d %H:%M:%S'))
 
         # Check that blacklised tags are not present anymore in the reduced
         # picture.
-        def lat(): return dest_image['Exif.GPSInfo.GPSLongitude'].value
+        def lat(): return dest_image['Exif.GPSInfo.GPSLongitude']
         self.assertRaises(KeyError, lat)
 
-        def long(): return dest_image['Exif.GPSInfo.GPSLatitude'].value
+        def long(): return dest_image['Exif.GPSInfo.GPSLatitude']
         self.assertRaises(KeyError, long)
 
     def test_metadata_osize_nopublish(self):
@@ -158,22 +169,20 @@ class TestGenerators(LazygalTestGen):
         img_path = self.add_img(self.source_dir, 'md_filled.jpg')
 
         # Add some metadata
-        source_image = pyexiv2.ImageMetadata(img_path)
-        source_image.read()
+        source_image = GExiv2.Metadata(img_path)
         dummy_comment = 'nice photo'
         source_image['Exif.Photo.UserComment'] = dummy_comment
-        source_image.write()
+        source_image.save_file()
 
         # Generate album
         dest_dir = self.get_working_path()
         self.album.generate(dest_dir)
 
         dest_img_path = os.path.join(dest_dir, 'md_filled_small.jpg')
-        dest_image = pyexiv2.ImageMetadata(dest_img_path)
-        dest_image.read()
+        dest_image = GExiv2.Metadata(dest_img_path)
 
         # Check that metadata is not here for reduced pictures.
-        def com(): return dest_image['Exif.Photo.UserComment'].value
+        def com(): return dest_image['Exif.Photo.UserComment']
         self.assertRaises(KeyError, com)
 
     def test_resize_rotate_size(self):
@@ -183,10 +192,9 @@ class TestGenerators(LazygalTestGen):
 
         norotate_path = self.add_img(self.source_dir, 'norotate.jpg')
         torotate_path = self.add_img(self.source_dir, 'torotate.jpg')
-        torotate = pyexiv2.ImageMetadata(torotate_path)
-        torotate.read()
-        torotate['Exif.Image.Orientation'] = 8
-        torotate.write()
+        torotate = GExiv2.Metadata(torotate_path)
+        torotate['Exif.Image.Orientation'] = '8'
+        torotate.save_file()
 
         # Generate album
         dest_dir = self.get_working_path()
@@ -206,8 +214,7 @@ class TestGenerators(LazygalTestGen):
         dest_dir = self.get_working_path()
         self.album.generate(dest_dir)
 
-        self.assertEqual(os.path.isfile(os.path.join(dest_dir, 'index.xml')),
-                         True)
+        self.assertTrue(os.path.isfile(os.path.join(dest_dir, 'index.xml')))
 
     def test_dirzip(self):
         config = lazygal.config.LazygalConfig()
@@ -219,9 +226,96 @@ class TestGenerators(LazygalTestGen):
         dest_dir = self.get_working_path()
         self.album.generate(dest_dir)
 
-        self.assertEqual(os.path.isfile(os.path.join(dest_dir, 'src.zip')),
-                         True)
+        self.assertTrue(os.path.isfile(os.path.join(dest_dir, 'src.zip')))
 
+    def test_filter_by_tag(self):
+        config = lazygal.config.LazygalConfig()
+        config.set('webgal', 'filter-by-tag', 'lazygal')
+        self.setup_album(config)
+
+        # good pictures will be pushed on the destination, false pictures
+        # should be filtered out.
+        # The sub-directory 'subfalse' should not be created on the destination
+        # side, because it should be empty.
+        good_path         = self.add_img(self.source_dir, 'good.jpg')
+        good_path2        = self.add_img(self.source_dir, 'good2.jpg')
+        false_path        = self.add_img(self.source_dir, 'false.jpg')
+        subgood           = self.setup_subgal('subgood', ['subgood.jpg', 'subfalse.jpg'])
+        subfalse          = self.setup_subgal('subfalse', ['subgood.jpg', 'subfalse.jpg'])
+        good_subdir_path  = os.path.join(self.source_dir, subgood.name, 'subgood.jpg')
+        false_subdir_path = os.path.join(self.source_dir, subfalse.name, 'subfalse.jpg')
+        good     = GExiv2.Metadata(good_path)
+        good2    = GExiv2.Metadata(good_path2)
+        false    = GExiv2.Metadata(false_path)
+        good_sd  = GExiv2.Metadata(good_subdir_path)
+        false_sd = GExiv2.Metadata(false_subdir_path)
+        good['Iptc.Application2.Keywords']     = 'lazygal'
+        good['Xmp.dc.subject']                 = 'lazygal2'
+        good.save_file()
+        good2['Iptc.Application2.Keywords']    = 'lazygalagain'
+        good2['Xmp.dc.subject']                = 'lazygal'
+        good2.save_file()
+        false['Iptc.Application2.Keywords']    = 'another_tag'
+        false.save_file()
+        good_sd['Iptc.Application2.Keywords']  = 'lazygal'
+        good_sd['Xmp.dc.subject']              = 'lazygal2'
+        good_sd.save_file()
+        false_sd['Iptc.Application2.Keywords'] = 'lazygal_lazygal'
+        false_sd['Xmp.dc.subject']             = 'lazygal2'
+        false_sd.save_file()
+
+        # generate album
+        dest_dir = self.get_working_path()
+        self.album.generate(dest_dir)
+
+        try:
+            self.assertTrue(os.path.isfile(os.path.join(dest_dir, 'good_thumb.jpg')))
+            self.assertTrue(os.path.isfile(os.path.join(dest_dir, 'good2_thumb.jpg')))
+            self.assertFalse(os.path.isfile(os.path.join(dest_dir, 'false_thumb.jpg')))
+            self.assertTrue(os.path.isfile(os.path.join(dest_dir, 'subgood', 'subgood_thumb.jpg')))
+            self.assertFalse(os.path.isfile(os.path.join(dest_dir, 'subfalse', 'subfalse_thumb.jpg')))
+            self.assertFalse(os.path.isfile(os.path.join(dest_dir, 'subfalse')))
+        except AssertionError:
+            print "\n contents of dest_dir : "
+            print os.listdir(dest_dir)
+            raise
+
+    def test_filter_and_dirzip(self):
+        config = lazygal.config.LazygalConfig()
+        config.set('webgal', 'dirzip', 'Yes')
+        config.set('webgal', 'filter-by-tag', 'lazygal')
+        self.setup_album(config)
+
+        # We need at least two pictures to display, because lazygal generates a
+        # zip archive only if there is more than one picture.
+        good_path = self.add_img(self.source_dir, 'good.jpg')
+        good_path2 = self.add_img(self.source_dir, 'good2.jpg')
+        false_path = self.add_img(self.source_dir, 'false.jpg')
+        good = GExiv2.Metadata(good_path)
+        good2 = GExiv2.Metadata(good_path2)
+        false = GExiv2.Metadata(false_path)
+        good['Iptc.Application2.Keywords'] = 'lazygal'
+        good['Xmp.dc.subject'] = 'lazygal2'
+        good.save_file()
+        good2['Iptc.Application2.Keywords'] = 'lazygalagain'
+        good2['Xmp.dc.subject'] = 'lazygal'
+        good2.save_file()
+        false['Iptc.Application2.Keywords'] = 'another_tag'
+        false.save_file()
+
+        # generate album
+        dest_dir = self.get_working_path()
+        self.album.generate(dest_dir)
+
+        try:
+            self.assertTrue(os.path.isfile(os.path.join(dest_dir, 'src.zip')))
+            self.assertTrue(os.path.isfile(os.path.join(dest_dir, 'good_thumb.jpg')))
+            self.assertTrue(os.path.isfile(os.path.join(dest_dir, 'good2_thumb.jpg')))
+            self.assertFalse(os.path.isfile(os.path.join(dest_dir, 'false_thumb.jpg')))
+        except AssertionError:
+            print "\n contents of dest_dir : "
+            print os.listdir(dest_dir)
+            raise
 
 class TestSpecialGens(LazygalTestGen):
 
@@ -237,7 +331,7 @@ class TestSpecialGens(LazygalTestGen):
         config.set('webgal', 'thumbs-per-page', 4)
         self.setup_album(config)
 
-        pics = [ 'img%d.jpg' % i for i in range(0, 9)]
+        pics = ['img%d.jpg' % i for i in range(0, 9)]
         source_subgal = self.setup_subgal('subgal', pics)
 
         self.album.generate(self.dest_path)
@@ -246,7 +340,7 @@ class TestSpecialGens(LazygalTestGen):
 
     def test_flatten(self):
         config = lazygal.config.LazygalConfig()
-        config.set('global', 'dir-flattening-depth', 1)
+        config.set('global', 'dir-flattening-depth', 0)
         self.setup_album(config)
 
         source_subgal = self.setup_subgal('subgal', ['subgal_img.jpg'])
@@ -257,10 +351,10 @@ class TestSpecialGens(LazygalTestGen):
     def test_flattenpaginate(self):
         config = lazygal.config.LazygalConfig()
         config.set('webgal', 'thumbs-per-page', 4)
-        config.set('global', 'dir-flattening-depth', 1)
+        config.set('global', 'dir-flattening-depth', 0)
         self.setup_album(config)
 
-        pics = [ 'img%d.jpg' % i for i in range(0, 9)]
+        pics = ['img%d.jpg' % i for i in range(0, 9)]
         source_subgal = self.setup_subgal('subgal', pics)
 
         self.album.generate(self.dest_path)
@@ -276,7 +370,7 @@ class TestSpecialGens(LazygalTestGen):
         """
         self.setup_album()
 
-        pics = [ 'img%d.jpg' % i for i in range(0, 2)]
+        pics = ['img%d.jpg' % i for i in range(0, 2)]
         source_subgal = self.setup_subgal('subgal', pics)
 
         # cp -ar to create an out-of-tree source dir with pics
@@ -336,14 +430,14 @@ class TestSorting(LazygalTestGen):
         for index, pic in enumerate(pics):
 
             img_path = self.add_img(subgal_path, pic)
-            img_exif = pyexiv2.ImageMetadata(img_path)
-            img_exif.read()
+            img_exif = GExiv2.Metadata(img_path)
             for tag in ('Exif.Photo.DateTimeDigitized',
                         'Exif.Photo.DateTimeOriginal',
                         'Exif.Image.DateTime',
-                       ):
-                img_exif[tag] = datetime.datetime(2011, months[index], 1)
-            img_exif.write()
+                        ):
+                img_exif[tag] = datetime.datetime(
+                    2011, months[index], 1).strftime('%Y:%m:%d %H:%M:%S')
+            img_exif.save_file()
 
         return subgal_path, pics
 
@@ -469,6 +563,7 @@ class TestSorting(LazygalTestGen):
         )
 
 
+
 if __name__ == '__main__':
     unittest.main()
 
